#include "proteinMMEnzyme2.h"

// #define DEBUG
// #define VERBOSE

using namespace std;

ProteinMMEnzyme2::ProteinMMEnzyme2(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme2::reaction_A1);
	fu1.pfunction = pr1;
	setFunction("A1", fu1);

	void (Protein::*pr2)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme2::reaction_R1);
	fu1.pfunction = pr2;
	setFunction("R1", fu1);

	void (Protein::*pr3)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme2::reaction_D1);
	fu1.pfunction = pr3;
	setFunction("D1", fu1);
	
	timer_R1 = 0;
	timer_D1 = 0;
	timer_D2 = 0;
}


bool ProteinMMEnzyme2::parseLine(char *linep)
{
	char  dummy[128];
	int idx;

	// read data into the variables
	if(sscanf(linep, "%s %d %s %lf %lf %s %lf %s %lf %s %lf", dummy, &idx, 
			  SubstratePool, &kon_S, &k_conv, ProductPool, &koff_P, ATPPool, &kon_ATP, ADPPool, &koff_ADP) != 11) {
		cerr << "ProteinMMEnzyme2::parseLine: not enough parameters for MMEnzyme2 line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs of the rates
	if((kon_S < 0.0) || (k_conv < 0.0) || (koff_P < 0.0) || (kon_ATP < 0.0) || (koff_ADP < 0.0)) {
		cerr << "ProteinMMEnzyme2::parseLine: negative rate constant in MMEnzyme2 line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   MMEnzyme: read: " << idx << endl;
	cout << "   MMEnzyme: read: " << SubstratePool << "  " << kon_S << endl;
	cout << "   MMEnzyme: read:              " << k_conv << endl;
	cout << "   MMEnzyme: read: " << ProductPool << "  " << koff_P << endl;
	cout << "   MMEnzyme: read: " << ATPPool << "  " << kon_ATP << endl;
	cout << "   MMEnzyme: read: " << ADPPool << "  " << koff_ADP << endl;
	#endif

	return(true);
}


void ProteinMMEnzyme2::verify_pools()
{
	if(!(Pools->count(string(SubstratePool)))) {
		cerr << "ProteinMMEnzyme2::verify_pools: pool "<<SubstratePool<<" not defined" << endl;
		exit(-3);
	}
	SubstratePoolp = getPool(SubstratePool);
	
	if(!(Pools->count(string(ProductPool)))) {
		cerr << "ProteinMMEnzyme2::verify_pools: pool "<<ProductPool<<" not defined" << endl;
		exit(-3);
	}
	ProductPoolp = getPool(ProductPool);
	
	if(!(Pools->count(string(ATPPool)))) {
		cerr << "ProteinMMEnzyme2::verify_pools: pool "<<ATPPool<<" not defined" << endl;
		exit(-3);
	}
	ATPPoolp = getPool(ATPPool);
	
	if(!(Pools->count(string(ADPPool)))) {
		cerr << "ProteinMMEnzyme2::verify_pools: pool "<<ADPPool<<" not defined" << endl;
		exit(-3);
	}
	ADPPoolp = getPool(ADPPool);
}



// ++++++++++++++ internal reaction ++++++++++++++++++++++++

// A1: association of the substrate
// product and ADP must have unbound before
void ProteinMMEnzyme2::reaction_A1(Protein *p) {
	if(!bs_S && !bs_P && !bs_ADP) {
		if(SubstratePoolp->take_out(kon_S)) {
			bs_S = true;
			writeInternals();
		}
	}
}

// A2: ATP uptake
void ProteinMMEnzyme2::reaction_A2(Protein *p) {
	if(bs_S && !bs_ATP && !bs_ADP) {
		if(ATPPoolp->take_out(kon_ATP)) {
			bs_ATP = true;
			writeInternals();
		}
	}
}

// R1: conversion of substrate in bs_S into product in bs_P
void ProteinMMEnzyme2::reaction_R1(Protein *p) {
	if(bs_S && bs_ATP) 
	{
		switch(timer_R1) 
		{
			case 0:  // initialize timer, start conversion
				double t0 = 1.0 / (k_conv * delta_t);
				timer_R1 = static_cast<unsigned long int>(exp_ran(t0));
				
				bs_P = true;
				writeInternals();
				break;
				
			case 1:   // conversion done
				bs_S = false;
				bs_ATP = false;
				bs_ADP = true;
				timer_R1 = 0;
				writeInternals();
				break;
				
			default:    // count down
				timer_R1--;
		}
	}
}


// D1: dissociation of the final product
void ProteinMMEnzyme2::reaction_D1(Protein *p) 
{
	if(!bs_S && bs_P) 
	{
		switch(timer_D1) 
		{
			case 0:  // initialize timer
				double t0 = 1.0 / (koff_P * delta_t);
				timer_D1 = static_cast<unsigned long int>(exp_ran(t0));
				break;
				
			case 1:   // release!
				bs_P = false;
				ProductPoolp->put_back();
				timer_D1 = 0;
				writeInternals();
				break;
				
			default:    // count down
				timer_D1--;
		}
	}
}


// D2: ADP release
void ProteinMMEnzyme2::reaction_D2(Protein *p) 
{
	if(bs_ADP) 
	{
		switch(timer_D2) 
		{
			case 0:  // initialize timer
				double t0 = 1.0 / (koff_ADP * delta_t);
				timer_D2 = static_cast<unsigned long int>(exp_ran(t0));
				break;
				
			case 1:   // release!
				bs_ADP = false;
				ADPPoolp->put_back();
				timer_D2 = 0;
				writeInternals();
				break;
				
			default:    // count down
				timer_D2--;
		}
	}
}
		


std::string ProteinMMEnzyme2::listInternals() {
	return(string("#time\tbs_S\tbs_P"));
}


void ProteinMMEnzyme2::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << bs_S << "\t" << bs_P << "\n";
		internalsOutFile.close();
	}
}
