//	Vesimulus 2.0: molecular stochastic simulation engine for metabolic systems
//	
//	developed from:
//	V1.x: BSc project of Florian Lauck
//	
//	main.cpp 

#include "main.h"

using namespace std;


//! main() for the whole simulation run: parse commandline, init the setup, and fire off the simulation
int main(int argc, char* argv[])
{
	// fixed size of the integration timestep
	double timestep;
	
	// init RNG
	init_ran();
	
	//! Command line args:
	//! - 1:  simulation length in secs
	//! - 2:  output interval in secs
	//! - 3:  time step in micro seconds (something like 1..10 for fast processes)
	//! - 4:  parameter file
	//! - 5:  optional name of basename of output parameter file, if none is given, 
	//!       output of pools levels etc goes to the console (stdout)
	//! - 6:  flag for output of internals (value does not matter)
	if((argc < 5) || (argc > 7))
	{
		cerr << "Vesimulus 2.0" << endl;
		cerr << "version of " <<
		#include "version.h"
		<< endl;
		cerr << "\nusage: " << argv[0] << " <time of simulation in seconds> <output intervall in sec> <timestep in micro secs> <parameter file> [<output file basename> [<output of internals: internals>]]\n" << endl;
		return(-1);
	}
	
	// read timestep from commandline
	if((sscanf(argv[3], "%lf", &timestep) != 1) || (timestep <= 0))
	{
		cerr << "### Error: No a valid timestep: " << argv[3] << " ###" << endl;
		return(-1);
	}
	// convert to micro seconds
	timestep *= 1e-6;
	
	// read simulation duration from commandline and convert into number of timesteps
	double runsec;
	if((sscanf(argv[1], "%lf", &runsec) != 1) || (runsec <= 0.0))
	{
		cerr << "### Error: Not a valid simulation length: " << argv[1] << " ###" << endl;
		return(-1);
	}
	unsigned long int runs = static_cast<long int>(runsec / timestep + 0.5);
	
	// read output interval from commandline and convert to number of timesteps
	double logsec;
	if((sscanf(argv[2], "%lf", &logsec) != 1) || (logsec <= 0.0) || (logsec > runsec))
	{
		cerr << "### Error: Not a valid output interval: " << argv[2] << " ###" << endl;
		return(-1);
	}
	unsigned long int logs = static_cast<long int>(logsec / timestep + 0.5);
	
	// check if input file is readable
	cout << "Check for input file " << argv[4] << endl;
	FILE *infile;
	if(!(infile = fopen(argv[4], "r"))) {
		cerr << "### Error: cannot open parameter file " << argv[4] << " ###" << endl;
		return(-1);
	}
	fclose(infile);
	
	// create new simulation
	Simulation NewSimulation;
	
	// --- now for the optional args ---
	// Read file basename for output, otherwise stdout (simulation will handle this)
	if ((argc >= 6))
	{
		cout << "Set output to file " << string(argv[5]) + "_pools.dat" << endl;
		NewSimulation.setOutputFilename(string(argv[5]) + "_pools.dat");
		NewSimulation.setOutputConsole("none");
		NewSimulation.setOutputProgress(string(argv[5]) + "_progress.txt");
	}
	else {
		NewSimulation.setOutputConsole("particles");
	}
	
	// if there is any arg[6] (non-empty string) log protein internals
	if((argc == 7))
	{
		if(string(argv[6]) != "")
		{
			cout << "main::main: Logging internal protein states to " << argv[5] << endl;
			NewSimulation.setOutputBS(string(argv[5]));
		}
	}
	
	// set a few more things
	NewSimulation.setTimestep(timestep);
	NewSimulation.setLogstep(logs);
	NewSimulation.setNumberOfSteps(runs);

	// parse input file to set up system and ...
	NewSimulation.parseFile(argv[4]);
	
	// ... run it
	NewSimulation.run();
	
	// Clean up (not really necessary, the OS could do that, too..)
	exit_ran();
	
	// bye bye!
	return(0);
}
