// Definitions for the automated inclusion into the parser
// LABEL:		bc1dimer2
// CLASSNAME:	ProteinBC1Dimer2
// 
#ifndef ProteinBC1Dimer2_h
#define ProteinBC1Dimer2_h
#include<string>
#include<stdlib.h>
#include<iostream>
#include<string>

#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"

#define MAX_POOLNAME_LENGTH 128

//! Class that represents bc1 protein complex of the vesicle, Version 2.

/*!
This class represents the second version of the cytochrome bc1 complex in 
its dimeric form, which transfers protons to the inside of the vesicle. 
It takes the 2 electron proton pairs from a quinol molecule and transfers 
both protons to the inside. One electron is passed via the Rieske 
(iron-sulphur, FeS) protein  to a waiting cytochrome c2 on the c2 binding site. 
The other electron and a proton from the outside are transfered to a 
(semi)quinone on the Qi site. The emptied Q can directly pass from the Qo 
to the Qi site with a given probability.

Binding sites: Q0 to dock Quinol, Qi to dock Quinone, c2 to dock the 
cytochrome c2, and HO for the protons released into the vesicle.

Electron numbers at Qo, Qi, c1, c2, FeS, bL. bH

Proton number at Qi

Position of the FeS unit: FeSb (=FeS attached to cyt b)

Pools: Q, QH2, c2red, c2ox, H+out, H+in, c1ox

Order of parameter in the setup file:
QH2onPool		kon_QH2_Qo
QoffPool		koff_Q_Qo
  ---  			k_Q_QoQi
QonPool			kon_Q_Qi
QH2offPool		koff_QH2_Qi
  ---			k_QH2_QiQo
c2oxPool		kon_c2ox_c1
c2redPool		koff_c2red_c1
HoutsidePool	kon_H_Qi
HinsidePool		koff_H_Qo
  ---			k_e_QoFeS
  ---			k_FeS_bc
  ---			k_FeS_cb
  ---			k_e_FeSc1
  ---   		k_e_c1c2
  ---			k_e_QobL
Obs_DeltaPhi	k_e_bLbH
  ---			Phi0_e_bLbH
  ---			k_e_bHQi
*/

class ProteinBC1Dimer2 : public Protein
{
	public:
	
	//! Constructor
	ProteinBC1Dimer2(const std::string name, const double &timestep);
	//! Destructor
	~ProteinBC1Dimer2();
	
	
	//! Parser for setup line
	bool parseLine(char *linep);
	
	//! @name some diagnostics / observables
	//@{
	//! get the number of oxidized c1, i.e., without an electron
	long int get_c1ox() { return static_cast<long int>(2 - reg_e_c1_1 - reg_e_c1_2);}	
	//@}
	
	private:
	
	//! @name internal functions (reactions) of the protein
	//@{
	//! binds a QH2 at either of the Qo sites
	void R_A1(Protein *);
	//! unbinds a Q from Qo to the adjacent Qi or to the pool, two indep. 
	//! reactions (timers!)
	void R_D1(Protein *);
	//! binds a proton from the outside
	void R_A2(Protein *);
	//! unbinds two proton to the vesicle inside
	void R_D2(Protein *);
	//! binds a Q at Qi
	void R_A3(Protein *);
	//! unbind QH2 at Qi to adjacent Qo or to the pool
	void R_D3_1(Protein *);
	void R_D3_2(Protein *);
	//! binds a c2 to one of the c2-binding sites
	void R_A4(Protein *);
	//! unbinds c2
	void R_D4_1(Protein *);
	void R_D4_2(Protein *);
	//! e- transfer Qo -> FeS_b
	void R_R1(Protein *);
	//! FeS-head movement cyt b -> cyt c and back
	void R_R2a_1(Protein *);
	void R_R2b_1(Protein *);
	void R_R2a_2(Protein *);
	void R_R2b_2(Protein *);
	//! e- transfer FeS_c -> c1
	void R_R3(Protein *);
	//! e- transfer c1 -> bound c2ox
	void R_R4(Protein *);
	//! transfer of the second e- from Qo -> bL
	void R_R5(Protein *);
	//! e- transfer bL -> bH, rate reduced with dPhi
	void R_R6(Protein *);
	//! e- transfer bL -> Qi
	void R_R7(Protein *);
	//@}
	
	//! Defs of the internal registers and binding sites.
	//! Two copies each, it's a dimer...
	//! (registers = int,  binding sites = bool)
	// @{
	//! Qo Binding site for quinol QH2 -> Q
	bool bs_Qo_1, bs_Qo_2;
	//! Qi Binding site for quinone Q -> QH2
	bool bs_Qi_1, bs_Qi_2;
	//! cytochrome c2 binding site at the Rieske protein
	bool bs_c2_1, bs_c2_2;
	//! bs for protons to the inside
	bool bs_H_1, bs_H_2;
	//! position of the FeS unit (true = next to cyt b)
	bool FeS_b_1, FeS_b_2;
	//! does the FeS block the proton release channel?
	bool FeS_open_1, FeS_open_2;
	//! number of e- and H+ still to unload on the Qo
	int reg_e_Qo_1, reg_e_Qo_2;
	int reg_H_Qo_1, reg_H_Qo_2;
	//! number of e- and H+ already on the Qi
	int reg_e_Qi_1, reg_e_Qi_2;
	int reg_H_Qi_1, reg_H_Qi_2;
	//! electron on the c1
	int reg_e_c1_1, reg_e_c1_2;
	//! electron on the c2
	int reg_e_c2_1, reg_e_c2_2;
	//! electron on the FeS subunit
	int reg_e_FeS_1, reg_e_FeS_2;
	//! electron on the bL heme
	int reg_e_bL_1, reg_e_bL_2;
	//! electron on the bH heme
	int reg_e_bH_1, reg_e_bH_2;
	// @}
	
	//! @name Char[]s for the pool and observable names (see length above)
	//! These pools only exist once, though it's a dimer.
	//@{
	char QoffPool[MAX_POOLNAME_LENGTH], 
		QH2offPool[MAX_POOLNAME_LENGTH], 
		QonPool[MAX_POOLNAME_LENGTH], 
		QH2onPool[MAX_POOLNAME_LENGTH], 
		c2redPool[MAX_POOLNAME_LENGTH], 
		c2oxPool[MAX_POOLNAME_LENGTH], 
		HoutsidePool[MAX_POOLNAME_LENGTH], 
		HinsidePool[MAX_POOLNAME_LENGTH];
	char DeltaPhiObs[MAX_POOLNAME_LENGTH];
	char c1oxPool[MAX_POOLNAME_LENGTH];
	//@}

	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *QoffPoolp, *QH2offPoolp, *QonPoolp, *QH2onPoolp, *c2redPoolp, *c2oxPoolp, *HoutsidePoolp, *HinsidePoolp;
	Observable *DeltaPhiObsp;
	double kon_QH2_Qo, koff_Q_Qo, k_Q_QoQi, kon_Q_Qi, koff_QH2_Qi, k_QH2_QiQo, kon_c2ox_c1, koff_c2red_c1, kon_H_Qi, 
		   koff_H_Qo, k_e_QoFeS, k_FeS_bc, k_FeS_cb, k_e_FeSc1, k_e_c1c2, k_e_QobL, k_e_bLbH, Phi0_e_bLbH, k_e_bHQi;
	//@}
	
	//! timer variables
	unsigned long int timer_d3_1, timer_d3_2, timer_d4_1, timer_d4_2, timer_r2a_1, timer_r2a_2, timer_r2b_1, timer_r2b_2;
	
	//! verify that all neccessary pools are defined
	void verify_pools();

	//! @name For listing the internals
	// @{
	std::string listInternals();
	void writeInternals();
	// @}
};
#endif
