#include"proteinProtonateable.h"

// #define DEBUG
// #define VERBOSE

// if you want to enable output of the internals, uncomment the following define
// #define ENABLE_LOGGING


using namespace std;

ProteinProtonateable::ProteinProtonateable(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	// initial conditions: no protons bound
	occupied_sites = 0;
	
	// register function
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinProtonateable::reaction1);
	fu1.pfunction = pr1;
	setFunction("r1", fu1);
}


void ProteinProtonateable::verify_pools()
{
	if(!(Pools->count(string(ProtonPool)))) {
		cerr << "ProteinProtonateable::verify_pools: pool "<<ProtonPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(HProteinPool)))) {
		cerr << "ProteinProtonateable::verify_pools: pool "<<HProteinPool<<" not defined" << endl;
		exit(-3);
	}
	
	ProtonPoolp   = getPool(ProtonPool);
	HProteinPoolp = getPool(HProteinPool);
}



bool ProteinProtonateable::parseLine(char *linep)
{
	// double rate0, rate1, rate2;
	char  dummy[128]; // , pool0[128], pool1[128];
	int num_of_proteins;

	// read date into the variables
	if(sscanf(linep, "%s %d %s %lf %lf %lf %s", dummy, &num_of_proteins, 
			  ProtonPool, &numOfResidues, &pK, &dpK, HProteinPool) != 7) {
		cerr << "ProteinProtonateable::parseLine: not enough parameters for protonateable line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs
	if((pK < 0.0) || (dpK < 0.0)) {
		cerr << "ProteinProtonateable::parseLine: negative rate constant in protonateable line >>" << linep << "<<" << endl;
		return(false);
	}
	// Is there at least one protonatable residue?
	if(numOfResidues < 1.0) {
		cerr << "ProteinProtonateable::parseLine: Number of Residues < 1 in protonateable line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef VERBOSE			
	cout << "------------------" << endl;
	cout << "   protonateable: read: " << num_of_proteins << endl;
	cout << "   protonateable: read: " << ProtonPool << "  " << numOfResidues << endl;
	cout << "   protonateable: read: " << pK << "  " << dpK << endl;
	cout << "   protonateable: read: " << HProteinPool << endl;
	#endif

	return(true);
}


std::string ProteinProtonateable::listInternals() {
	#ifdef ENABLE_LOGGING
	return(string("#time\toccupied_sites"));
	#else
	outputInternals = false;
	return(string("# LOGGING DISABLED, define ENABLE_LOGGING in the sources"));
	#endif
}


void ProteinProtonateable::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << occupied_sites << "\n";
		internalsOutFile.close();
	}
}

// ++++++++++++++ internal reactions ++++++++++++++++++++++++

void ProteinProtonateable::reaction1(Protein *p)
{
	// get pool volume
	double H_volume = ProtonPoolp->getVolume();
	
	// get number of free H+
	double H_free = (double)(ProtonPoolp->getInside());
	double H_tot = H_free + occupied_sites;
	double diff = 0.0;
	double H_temp;
	double occupied_save = occupied_sites;
	
	if(H_tot > 0) 
		{
		// solve for avg number of bound H+ (Newton)
		double n_min = 0.0;
		double n_max = (H_tot > ((double)numOfResidues)) ? ((double)numOfResidues) : H_tot;
		H_temp = n_max / 2.0;
		
		while(true) 
		{
			diff = log((H_tot - H_temp) / (H_volume*0.6)) / log(10.0) + pK + dpK*log((numOfResidues - H_temp) / (H_temp ));  // 0.6 for 1/nm3 -> mol/l
			
			if(fabs(diff) < 1.0e-2) {
				break;
			}
			
			if(diff >= 0.0) {
				n_min = H_temp;
			} else {
				n_max = H_temp;
			}
			
			H_temp = (n_max + n_min ) / 2.0;
		}
		
		// adapt number of bound H+, stochastically bind (or not) the last one
		long int oneLess = (long int)(H_temp);
		long int austausch;
		
		// set occupied_sites to int below avg. num H_temp
		if(oneLess > occupied_sites)
		{
			austausch = oneLess - occupied_sites;
			ProtonPoolp->take(austausch);
			HProteinPoolp->put_back(austausch);
			occupied_sites = oneLess;
		}
		
		if(oneLess < occupied_sites)
		{
			austausch = occupied_sites - oneLess;
			ProtonPoolp->put_back(austausch);
			HProteinPoolp->take(austausch);
			occupied_sites = oneLess;
		}
		
		// stochastically add remainder
		double fehltNoch = H_temp - (double)oneLess;
		if(lin_ran() < fehltNoch) {
			long int eins = 1L;
			ProtonPoolp->take(eins);
			HProteinPoolp->put_back(eins);
			occupied_sites++;
		}
		
		#ifdef ENABLE_LOGGING
		if(occupied_save != occupied_sites) {
			writeInternals();
		}
		#endif
	} 
}

