// Definitions for the automated inclusion into the parser
// LABEL:		RC
// CLASSNAME:	ProteinRC
// 
#ifndef ProteinRC_h
#define ProteinRC_h

#include<string>
#include<stdlib.h>
#include<iostream>
#include<fstream>
#include<string>
#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"



//! Class that represents the Reaction Center.

/*!
It loads 2 proton-electron-pair to a quinone, the electrons are delivered by 2 cytochrome c2 
and the energy for the transfer is provided by an electronic excitation from the light harvesting complex. 
It has 2 binding sites: Q for Quinone, C for cytochrome C2.
Internal Values: "special pair" a pair of bacteriochlorophylls from where an electron is transferred to the Quinone.
"quinone": the reduction state of quinone: O: quinone (Q), 1: Semiquinone (QH), 2: Quinol (QH2)
The reaction center accesses several Pools:
Light-Harvesting-Pool, Q-Pool, QH2-Pool, H+Outside, Cytochrome c2 oxidized, Cytochrome c2 reduced
*/

class ProteinRC : public Protein
{
	
  public:
	
	//! Constructor, sets the initial states of the binding sites, internals ,...      
	ProteinRC(const std::string name, const double &timestep);
	//! Destructor
	~ProteinRC() { }
	
	//! Parse setup line
	bool parseLine(char *linep);
	
	//! @name Some diagnostics / hooks for observables
	//@{
	//! Get the number of electrons on the special pair
	long int get_SPe() { return static_cast<long int>(reg_SP);}
	//! Get the number of electron-proton pairs on the Qb
	long int get_QHe() { return static_cast<long int>(reg_QHe);}
	//! Get the number of oxidized c2 bound to the RC
	long int get_c2ox();
	//@}

	
  private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! Bind a Q molecule
	void reaction1(Protein *p);
	//! Transfer a Proton and an electron to the (semi)quinone
	void reaction2a(Protein *p);
	void reaction2b(Protein *p);
	//! Bind a Cytochrome c2
	void reaction3(Protein *p);
	//! Transfer an electron from a bound c2 to the special pair and unbind c2
	void reaction4(Protein *p);
	//! Unbind the QH2
	void reaction5(Protein *p);
	//@}
	
	//! Defs of the internal registers and binding sites.
	//! (registers = int,  binding sites = bool)
	// @{
	//! Binding site for Qb quinone
	bool bs_Q;
	//! Binding site for cytochrome c2
	bool bs_C;
	//! Electron on the special pair
	int reg_SP;
	//! Electron-proton-pairs and excess electrons on the QB quinone (has to be zero if no Q is bound)
	int reg_QHe;
	int reg_Qe;
	// @}
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char LHPool[MAX_POOLNAME_LENGTH], 
		 QPool[MAX_POOLNAME_LENGTH], 
		 QH2Pool[MAX_POOLNAME_LENGTH], 
		 c2redPool[MAX_POOLNAME_LENGTH], 
		 c2oxPool[MAX_POOLNAME_LENGTH], 
		 HoutsidePool[MAX_POOLNAME_LENGTH],
		 HInternalPool[MAX_POOLNAME_LENGTH];
	//@}
	
	//! Verify that all neccessary pools are defined
	void verify_pools();

	//! @name For listing the internals
	// @{
	std::string listInternals();
	void writeInternals();
	// @}
	
	//! @name Pointers to pools and variables for reaction constants to avoid their repeated lookup
	//@{
	Pool *LHPoolp, *QPoolp, *QH2Poolp, *c2redPoolp, *c2oxPoolp, *HoutsidePoolp, *HInternalPoolp;
	double Q_kon, LH_kon, Houtside_kon, c2red_kon, c2ox_koff, QH2_koff;
	//@}
	
	//! Timer variables for some of the reactions -- faster than the continuous lookup in the structure
	unsigned long int timer_r4, timer_r5;
};
#endif


