#ifndef RANDOM_H
#define RANDOM_H

//! Include file for random number generating functions


#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>


// Defines to select one of the following RNG algorithms (defined in the GSL):
// * maximally equidistributed combined Tausworthe generator by L'Ecuyer
#define RNG_TYPE gsl_rng_taus2
// * MT19937 generator of Makoto Matsumoto and Takuji Nishimura
// #define RNG_TYPE gsl_rng_mt19937



//! @name Initialization
// @{
//! Allocate and init RNG from system time
void init_ran();
//! Allocate and init RNG with fixed integer. seed=0 gives the default init from GSL.
void set_ran2(unsigned long int seed);
//! Clean up RNG
void exit_ran();
// @}


//! @name the different random distributions
// @{
//! Linear distribution from [0,1)
double lin_ran();

//! Exponential distribution acc. exp(-t/t0) for dissociation reactions.
//! t0 = 1/rate constant.
double exp_ran(double &t0);

//! Normal distributed random numbers acc. exp(-(t-t0)^2 / (2*sigma^2)).
//! when using this one watch out for negative return values (i.e., negative waiting times)!!!
double normal_ran(double &t0, double &sigma);

//! Linearly distributed integers from [0, n-1]
unsigned long int int_ran(unsigned long int n);
// @}

#endif /* RANDOM_H */
