#ifndef Simulation_h
#define Simulation_h

#include <iostream>
#include <string>
#include <list>
#include <map>
#include <stdlib.h>
#include <cstdlib>
#include <time.h>
#include <stdio.h>
#include <fstream>
#include <vector>
#include <list>

#include "random.h"
#include "pool.h"
#include "parameter.h"
#include "protein.h"
#include "observable.h"
// the next one contains all Protein and Observable classes currently used
#include "myProteinHeaders.include"


// maximal lengths of input line and protein/pool/observable names
#define MAX_INPUTLINE_LENGTH	2048
#define MAX_PROTEINNAME_LENGTH	128
/*!
This class contains methods to do a simulation of a vesicle specified in a datafile. 
This datafile is parsed and different output can be specified.
*/


class Simulation
{
  public:
  
	//! Struct to store information about upcoming changes of pool concentrations
	struct light_periode
	{
		std::string poolName;
		double time;
		long int particles;
		bool infinite;
	};
	
	//! Default constructor
	Simulation();
	//! Destructor
	~Simulation();

	
	//! @name General setup methods
	//@{
	//! Set the time step for the simulation
	void setTimestep(const double &dt) { delta_t = dt; }
	//! Sets the length of the simulation in units of timesteps
	void setNumberOfSteps(unsigned long int);
	//! Parse a setup file for a simulation
	void parseFile(std::string);
	//! Run the simulation (after the setup)
	void run();
	//@}
	
	//! @name Output options
	//@{
	//! Set the output interval in units of timesteps
	void setLogstep(unsigned long int);
	//! Set the filename for particle output
	void setOutputFilename(std::string);
	
	// sets filename for concentration output
	// void setOutputConcentration(std::string);
	// 
	//! sets the filename for binding site and internal state output of RC to string
	void setOutputBS(std::string);
	//! sets kind of console output: none, particles, concentrations
	void setOutputConsole(std::string);
	//! sets the name of the progress file
	void setOutputProgress(std::string);
	//@}
	
  private:
  
  	//! Size of the simulation time step
  	double delta_t;
	//! Number of simulation steps: 1 step = delta_t
	unsigned long int num_of_steps;
	//! value for output frequency: every 'logstep' * delta_t
	unsigned long int logstep;
	
	//! list to store the proteins
	std::list<Protein*> Proteins;
	//! Get a random position in the protein list for insertion of the next protein
	std::list<Protein*>::iterator getNextPosition();

	
	//! @name Maps to store Pools and Observables
	//@{
	std::map<std::string, Pool*> Pools;
	std::list<std::string> PoolNames;
	std::map<std::string, Observable*> Observables;
	std::list<std::string> ObservableNames;
	//@}
	
	//! @name Lists and maps to store data for concentration changes
	//@{
	std::map<std::string, double> ChangesNames;
	std::list<light_periode> Light;
	//@}
	
	//! @name Output options
	// @{
	//! Filename for output (when not to console)
	std::string output_file;
	//! Filename for output of internal variables
	std::string output_BS;
	//! What is written to the console? none, particles, concentrations
	std::string output_console;
	//! Name of progress file (gives progress in percent)
	//! Can be switched on via compiler flag -DHAVE_PROGRESS_FILE
	std::string output_progress;
	// @}
};
#endif
