#ifndef Protein_h
#define Protein_h

#include<string>
#include<map>
#include<stdlib.h>
#include<iostream>
#include <fstream>

#include "pool.h"
#include "parameter.h"
#include "observable.h"

// forward declaration of Observable
class Observable;

// Max. length of pool names in the setup file (actually, it's 127 chars)
#define MAX_POOLNAME_LENGTH 128

//! Base class for the implementation of a protein.

//! Every protein is a subclass of this class. One object of a protein class represents one protein. 
//! It is defined from the binding sites, registers (= states of active sites), and the functions acting on them.

class Protein
{
	public:
	
	//! Struct that represents a function of a protein, 
	//! contains a pointer to the reaction and a timer which is currently unused.
	//! (Timers realized via dedicated variables are more efficient).
	struct function                                  
	{
		void (Protein::* pfunction)(Protein *);
		// unsigned long int time;
	};
	
	//! default constructor, give the protein a label and set the time step
	Protein(const std::string name, const double &timestep);

	//! copy constructor -- currently unused
	// Protein(const Protein& aProtein);
	
	//! Destructor
	//! (should be virtual to cope with the virtual private functions 
	//! listInternals() and writeInternals()???)
	~Protein();
	
	//! trigger all reactions of this protein for one timestep
	void timestep();
	//! set the timer of a protein
	void setTimer(unsigned long int);
	
	//! read the protein's label
	std::string readMyName() {
		return(myName);
	}
	
	//! @name Interfacing to the outside pools and observables
	//@{
	//! Tell the protein about the pointer to the map of external pools. These pools live in the simulation.
	// void setPools(std::map<std::string, Pool*>*);
	//! Tell the protein about the pointers to the maps of external pools 
	//! and observables. These live in the simulation.
	void setPoolsObservables(std::map<std::string, Pool*>*, std::map<std::string, Observable*>*);
	//! get access to a certain external pool
	Pool* getPool(std::string);
	//! get access to a certain global observable
	Observable* getObservable(std::string);
	//@}
	
	//!@name Working with internals
	//@{
	//! add a pointer to a reaction of the protein to the relating map
	void setFunction(std::string, function);
	// set the reaction constant for the (internal) reaction "string"
	// void setReactionConstant(std::string, double);
	// read the value of the reaction constant for "string"
	// double getReactionConstant(std::string);
	//@}
	
	//! initialize output of the internals
	void initInternalOutput(const std::string);
	
	protected:
	
	//! String label for this protein
	std::string myName;
	//! Internal time counter of the protein to keep track of the simulation time
	unsigned long int timer;
	//! Time step of the simulation, set at protein construction
	double delta_t;
	
	//! std::map containing the functions defining the protein
	// @todo Replace map with list, put function name into function struct
	std::map<std::string, function> Functions;
	//! Pointer to map of <name of Pool, pointer to Pool>. 
	//! This map is part of class Simulation and is used during initialization-
	std::map<std::string, Pool*>* Pools;
	//! Pointer to map of <name of Observable, pointer to Observable> for initialization.
	std::map<std::string, Observable*>* Observables;
	//! Check whether all neccessary pools and observables exist in the simulation's list of pools.
	//! Is virtual because the list of required pools is different for each protein type.
	virtual void verify_pools() { };
	//  names of the reaction constants plus their values
	// std::map<std::string, double> ReactionConstants;
	
	//! @name For output of the internals
	// @{
	//! flag and filename for output of internals after each change
	bool outputInternals;
	std::string internalFileName;
	std::ofstream internalsOutFile;
	
	//! Create a heading line in the output file with the internals' names.
	//! If there are no internals to list
	virtual std::string listInternals();
	//! Write out internals, when anything has changed, triggered from the individual functions.
	virtual void writeInternals();
};
#endif

